/*
 * DS18S20.c
 *
 * Created: 30.11.2012 19:23:17
 *  Author: Chris
 */ 

#include "DS18S20.h"
#include <util/atomic.h>

#include <util/crc16.h>


static void ow_null()
{
	OW_PORT &= ~(1<<OW_PIN);	//Pullup aus bzw Port auf Low
	OW_DDR |= (1<<OW_PIN);		//OW_PIN als Ausgang auf low
}


static void ow_eins()
{
	OW_DDR &= ~(1<<OW_PIN);		//OW_PIN als Eingang
	OW_PORT |= (1<<OW_PIN);		//Pullup an bzw Port auf High
}


static void ow_write_bit(uint8_t bitwert)
{
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		ow_null();		//Beginn des Slots
		_delay_us(1);
		if (0!=bitwert)	//bei logisch 1, Leitung sofort (nach 1s) wieder auf high;
		{
			ow_eins();
		}
	}
	
	_delay_us(60);	//langer delay bis Ende des TimeSlots
	
	ow_eins();		//jetzt wieder auf high
	_delay_us(1);	// warten, bevor nchstes Bit geschickt wird
}


static void ow_write_byte(uint8_t bytewert)
{
	uint8_t i;
	
	for (i=0; i<8; i++)
	{
		//ow_wr_bit( ( bytewert & (1<<i) ) );	// (bytewert & (1<<i) ) ist das i-te Bit (0 = LSB)
		
		ow_write_bit( bytewert & 0x01);
		bytewert = bytewert >> 1;
	}
	
}



static uint8_t ow_read_bit()
{
	uint8_t input;
	
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		ow_null();	//auf null setzen
		_delay_us(1);	//kurze Pause
		ow_eins();	//auf eins setzen / Pullup an
		
		_delay_us(10);	//innerhalb von max. 15 s samplen
		input = ( OW_INPUT & (1<<OW_PIN) );	//Zustand der Leitung wird gespeichert
	}
	
	
	_delay_us(55);	//langer delay bis Ende des TimeSlots
	
	return (input);		//bergabe des eingelesenen Bits
}



static uint8_t ow_read_byte()
{
	uint8_t i;
	uint8_t wert=0;
	
	for (i=0; i<8; i++)
	{
		if (ow_read_bit())	wert |= (1<<i);		//Wenn das neue Bit eine 1 ist, wird es entsprechend in 'wert' gesetzt
	}
	
	return wert;	//komplettes Byte zurckgeben
}






//////////////////////////////////////////////////////////////////////////
// globale Funktionen


 uint8_t ow_reset()		//Lst Reset aus und gibt 0 zurck, falls (mindestens) ein Slave antwortet
{
	uint8_t slave_anwesend;
	
	ow_null();			//Reset Puls
	
	_delay_us(480);		// Wartezeit
	
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		ow_eins();		//Leitung wieder high und als Eingang
		
		_delay_us(65);	//dann kommt die Antwort vom Sensor
		slave_anwesend = ( OW_INPUT & (1<<OW_PIN) );		//wenn Leitung auf 0, wird eine 0 gespeichert und andersrum
	}
	
	_delay_us(480-65);		// Delay bis Ende Reset-Vorgang
	
	//Reset Vorgang abgeschlossen
	
	return (slave_anwesend);		//0 = Sensor angeschlossen, sonst kein Sensor angeschlossen
}



#ifdef USE_READ_ROM
// Rckgabewert:	0 = Erfolg;		1 = kein Slave,		2 = CRC-Fehler
// in data[] mssen 8 Bytes Platz haben
uint8_t ow_read_rom(uint8_t* data)
{
	uint8_t i;
	
	//Start mit Reset und Abfrage, ob berhaupt ein Slave da ist
	if ( ow_reset() )	//dann ist kein Slave da
	{
		return 1;	// Fehler
	}
	
	// Abfrage-Befehl senden: "READ ROM" = 0x33
	ow_write_byte(0x33);
	
	// Antwort einlesen: 8 Byte = 64 Bit Identifier ins
	// globale Array rom_c[.]
	for (i=0; i<8; i++)
	{
		data[i] = ow_read_byte();
	}
	
	uint8_t crc = 0;
	
	for (i=0; i<8; i++)
	{
		crc = _crc_ibutton_update(crc, data[i]);
	}
	
	if (crc == 0)
	{
		return 0;
	}
	else
	{
		return 2;
	}
}
#endif


// startet die Temperaturmessung fr einen am Bus angeschlossenen Sensor (arbeitet mit SkipROM Befehl)
void DS18B20_start_measure()
{
	ow_reset();
	ow_write_byte(0xCC);	// skip-ROM Befehl
	
	// Befehl: Start Conversion an DS1820
	ow_write_byte(0x44);
	
	// jetzt muss gewartet werden, bis Messung fertig ist (ca. 750 ms)
}


// sensor_type gibt an, ob es sich um einen DS18 B 20 (0x28) oder einen DS18 S 20 (0x10) handelt
// entsprechend wird die Temperatur berechnet
// Rckgabewert: Temperatur * 10	--> z.B. 123 = 12,3 C
int16_t DS18B20_read_temp(uint8_t sensor_type)
{
	uint8_t ds_data[9];
	uint8_t i;
	int16_t temperature;
	
	if (ow_reset())		// kein Sensor/Slave antwortet
	{
		return OW_NO_SENSOR;
	}
	
	ow_write_byte(0xCC);	// skip-ROM Befehl
	
	ow_write_byte(0xBE);	// read Scratchpad
	
	for (i=0; i<9; i++)
	{
		ds_data[i] = ow_read_byte();
	}
	
	temperature = (ds_data[1] << 8) | ds_data[0];
	
	
	if (sensor_type == 0x10)	// DS18S20
	{
		// 9bit; temperature = 2*Messwert
		temperature *= 5;	// --> 10facher Wert
	}
	else	// 0x28 = DS18B20; default ebenso DS18B20, deswegen keine 'else if'
	{
		// 12bit; temperature = 16*Messwert
		temperature *= 10;
		temperature /= 16;		// --> 10facher Wert
	}
	
	// CRC-Check
	
	uint8_t crc = 0;
	
	for (i=0; i<9; i++)
	{
		crc = _crc_ibutton_update(crc, ds_data[i]);
	}
	
	if (crc == 0)	// Daten ok?
	{
		return temperature;
	}
	else	// Daten fehlerhaft
	{
		return OW_CRC_ERROR;
	}
}